"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

WindowView subclass: #BrowserWindowView
	instanceVariableNames: 'ebox'
	classVariableNames: 'windows'
	poolDictionaries: ''
	category: 'GUI-Framework'!

!BrowserWindowView methodsFor: 'initialization'!

buildPopupMenuItems
    super buildPopupMenuItems.
    popupMenuItems
		section: 'Filing' add: 'Save snapshot' sending: #snapshotCallback to: self;
		section: 'Closing' add: 'Close window' sending: #close: to: self;
		section: 'Windows' add: 'Open Workspace...' sending: #openWorkspaceWindow to: self;
		section: 'Windows' add: 'Open Namespaces Window...' sending: #openNamespacesWindow to: self;
		section: 'Windows' add: 'Open Namespace Window...' sending: #promptOpenNamespaceWindow to: self;
		section: 'Windows' add: 'Open Class Window...' sending: #promptOpenClassWindow to: self;
		section: 'Windows' add: 'Open Method Window...' sending: #promptOpenMethodWindow to: self;
		section: 'Windows' add: 'Inspect this window...' sending: #openInspectorForSelf to: self;
		section: 'Quitting' add: 'Quit...' sending: #quitCallback to: self.
    ^popupMenuItems
!

buildWidgets    
	super buildWidgets.
	self hookupPopupMenuTo: wnd.
!

attachViews
	self child ifNotNil: [ :c | wnd add: c external ].
!
!

!BrowserWindowView methodsFor: 'events'!

openWindow: aWindow
	aWindow show.
	Primitive updateViews.
!

openWorkspaceWindow
	self openWindow: (WindowFactory buildWorkspaceWindow: nil).
!

openNamespacesWindow
	self openWindow: (WindowFactory buildNamespacesWindow).
!

findNamespace: aName
	| name |
	name := aName trimSeparators.
	name isEmpty ifTrue: [ ^nil ].
	Smalltalk withAllSubspacesDo: 
		[ :ns | 
		(ns name match: name) ifTrue: [ ^ns ]].
	^nil
!

promptOpenNamespaceWindow
	| pt |

    pt := Prompter 
		title: 'Open Namespace Window' 
		entry: 'Namespace: '
		buttons: (Prompter buttonsOkCancel).
    "pt entryText:."
	
    [ (pt run = Gtk gtkResponseOk) 
		ifTrue: 
			[ (self findNamespace: pt entryText)
				ifNotNil: 
					[ :ns |
					self openWindow: 
						(WindowFactory buildNamespaceWindow: ns). ]. ] ] 
		ensure: 
			[ pt destroy ].
!

findClass: aName
	| name |
	name := aName trimSeparators.
	name isEmpty ifTrue: [ ^nil ].
	name := name asGlobalKey.
	Smalltalk withAllSubspacesDo: 
		[ :ns | ns at: name ifPresent: [ :cls | ^cls ] ].
	^nil
!

promptOpenClassWindow
	| pt |

    pt := Prompter 
		title: 'Open Class Window' 
		entry: 'Class: '
		buttons: (Prompter buttonsOkCancel).
    "pt entryText:."
	
    [ (pt run = Gtk gtkResponseOk) 
		ifTrue: 
			[ (self findClass: pt entryText) 
				ifNotNil: 
					[ :cls |
					self openWindow: 
						(WindowFactory buildClassWindow: cls). ]. ] ]
		ensure: 
			[ pt destroy ].
!

findMethod: aString
	| sm cls sel err |
	sm := ReadStream on: aString.
	sm atEnd ifTrue: [ ^nil ].
	
	cls := (sm upToAll: '>>') trimSeparators.
	(cls isEmpty or: sm atEnd) ifTrue: [ ^nil ].
	sel := sm upToEnd.
	sel isEmpty ifTrue: [ ^nil ].
	
	cls := self findClass: cls.
	cls isNil ifTrue: [ ^nil ].
	
	^cls compiledMethodAt: sel asGlobalKey ifAbsent: [ nil ].
!

promptOpenMethodWindow
	| pt |

    pt := Prompter 
		title: 'Open Class Window' 
		entry: 'Class: '
		buttons: (Prompter buttonsOkCancel).
    "pt entryText:."
	
    [ (pt run = Gtk gtkResponseOk) 
		ifTrue: 
			[ (self findMethod: pt entryText)
				ifNotNil: 
					[ :cm | 
					self openWindow: 
						(WindowFactory buildMethodWindow: cm). ]. ] ]
		ensure: 
			[ pt destroy ].
!

openInspectorForSelf
	self openWindow: (WindowFactory buildInspectorWindow: self).
!
!
