"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

SimpleTreeNode subclass: #MethodTreeNode
	instanceVariableNames: 'parent children data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

MethodTreeNode subclass: #MetaMethodTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

MethodTreeNode subclass: #ProtocolMethodTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

MethodTreeNode subclass: #MethodMethodTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

TreeData subclass: #ClassData
	instanceVariableNames: 'displayClass root flatMode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'
!

WindowedView subclass: #ClassView
	instanceVariableNames: 'vbox ebox tree superLblBtn classLbl commentLbl commentSw instVarsLbl classVarsLbl treeMsgs showComments contextMenuItems'
	classVariableNames: 'ShowComments'
	poolDictionaries: ''
	category: 'Mumble-Main'!

MethodTreeNode methodsFor: 'accessing'!

label
	^self subclassResponsibility
!

isClassNode
	parent isNil ifTrue: [ self error: 'Class node? Don''t know' ].
	^parent isClassNode
!

category
	parent isNil ifTrue: [ ^nil ].
	^parent category
!
!

MetaMethodTreeNode methodsFor: 'accessing'!

label
	^data asString, ' methods'
!

isClassNode
	^(data = #class)
!
!

ProtocolMethodTreeNode methodsFor: 'accessing'!

label
	^data asString
!

category
	^data asString
!
!

MethodMethodTreeNode methodsFor: 'accessing'!

label
	^data selector
!
!

ClassData methodsFor: 'initialization'!

buildFlatList: aParent from: aClass
	| mths pts labelSort |

	"Sort the methodDictionary by protocol"
	mths := 
		(aClass methodDictionary ifNil: [ ^nil ])
			values asSortedCollection: 
				[ :a :b | a label < b label ].
	mths do:
		[ :mth | | mn |
		(mn := MethodMethodTreeNode new) data: mth.
		aParent add: mn ].
!

buildProtocolTree: aParent from: aClass
	| mths pts |
	
	"Sort the methodDictionary by protocol"
	mths := LookupTable new.
    (aClass methodDictionary ifNil: [ ^nil ]) 
		keysAndValuesDo: 
			[ :sel :cm |
			(mths
				at: cm methodCategory 
				ifAbsentPut: [ OrderedCollection new])
				add: cm. ].
	
	"Sort the protocols alphabetically"
	pts := mths keys asSortedCollection.
	pts do:
		[ :prot | | pn |
		(pn := ProtocolMethodTreeNode new) data: prot.
		aParent add: pn.
		
		"Sort the methods too"
		((mths at: prot) asSortedCollection: 
			[ :a :b | a selector < b selector ]) do:
			[ :mth | | mn |
			(mn := MethodMethodTreeNode new) data: mth.
			pn add: mn ] ].
!

buildTree
	| cnd ind |
	"Build the root, class and instance nodes"
	root := MetaMethodTreeNode new.
	(cnd := MetaMethodTreeNode new) data: #class.
	root add: cnd.
	(ind := MetaMethodTreeNode new) data: #instance.
	root add: ind.
	
	"Build the rest of the tree - depends on flatMode"
	(flatMode notNil and: [ flatMode ])
		ifTrue: 
			[ self buildFlatList: cnd from: displayClass class.
			self buildFlatList: ind from: displayClass. ]
		ifFalse:
			[ self buildProtocolTree: cnd from: displayClass class.
			self buildProtocolTree: ind from: displayClass. ].
!
!

ClassData methodsFor: 'accessing'!

flatMode
	^flatMode
!

flatMode: aBoolean
	flatMode := aBoolean
!
!

ClassData methodsFor: 'accessing'!

displayClass
    displayClass isNil ifTrue: [ displayClass := Object ].
    ^displayClass.!

displayClass: aClass
    displayClass := aClass.
	self buildTree.
    self changeState: #classChanged.
!
!

ClassData methodsFor: 'change update'!

getCols: aNode
    ^Array with: (aNode ifNil: [ '' ] ifNotNil: [ :a | a label ])
!

getNodes: aNode
	^(aNode ifNil: [ root ]) children.
!
!

ClassView methodsFor: 'initialization'!

initialize
	(treeMsgs := LookupTable new)
		at: #methodSelected put: #nodeSelected;
		at: #methodActivated put: #nodeActivated.
	self events
		add: #superBtnClicked;
		add: #newMethodClicked.
!

setup: aData stateChange: aStateChange nodesMsg: aNodesMsg colsMsg: aColsMsg
	self data: aData.
	self 
		stateChange: aStateChange
		updateWith: [ self classChanged ].
	tree 
		data: aData;
		nodesMsg: aNodesMsg;
		colsMsg: aColsMsg;
		stateChange: aStateChange
			updateWith: [ tree rebuildTree ].
		
	"Class and inst vars"
	classVarsLbl 
		data: aData;
		stateChange: aStateChange
			updateWith: 
				[ | vs |
				vs := classVarsLbl data displayClass classVarNames.
				classVarsLbl setLabel: 
					((vs isNil or: [ vs isEmpty ])
						ifTrue: [ '(no class variables)' ]
						ifFalse: [ vs fold: [ :a :b | a, ' ', b ] ] ) ].
	instVarsLbl 
		data: aData;
		stateChange: aStateChange
			updateWith: 
				[ | vs |
				vs := instVarsLbl data displayClass instVarNames.
				instVarsLbl setLabel: 
					((vs isNil or: [ vs isEmpty ]) 
						ifTrue: [ '(no instance variables)' ]
						ifFalse: [ vs fold: [ :a :b | a, ' ', b ] ] ) ].
	
	
!

buildSubsidiaryViews
	| sb |
	sb := PHScrollBox new.
	sb shadow: Gtk gtkShadowNone.
	super add: sb.
	
	classVarsLbl := PLabel new.
	sb add: classVarsLbl.

	sb := PHScrollBox new.
	sb shadow: Gtk gtkShadowNone.
	super add: sb.

	instVarsLbl := PLabel new.
	sb add: instVarsLbl.

	super add: (tree := PTree new).
	tree
		buildColumns: #('Method');
		nodesMsg: #getNodes: colsMsg: #getCols:.
!

buildWidgets
	| al hbox lbl sb |
	super buildWidgets.
	
    (ebox := GtkEventBox new) 
		setName: 'classEbox';
		show.
	
	(vbox := GtkVBox new: false spacing: 3) show.
	ebox add: vbox.
	
	(al := GtkAlignment new: 0.3 yalign: 0.5 xscale: 0.0 yscale: 0.0) show.
	vbox packStart: al expand: false fill: false padding: 0.
	
	(hbox := GtkHBox new: false spacing: 12) show.
	al add: hbox.
	
	(superLblBtn := GtkButton newWithLabel: '') 
		setRelief: Gtk gtkReliefNone;
		on: 'clicked' send: #superBtnCallback to: self;
		show.
	hbox packStart: superLblBtn expand: false fill: false padding: 0.
	
	(lbl := GtkLabel new: '>>') show.
	hbox packStart: lbl expand: false fill: false padding: 0.
	
	(classLbl := GtkLabel new: '') show.
	vbox packStart: classLbl expand: false fill: true padding: 0.

	(commentSw := GtkScrolledWindow new: nil vadjustment: nil) show.
	commentSw setPolicy: Gtk gtkPolicyAutomatic vscrollbarPolicy: Gtk gtkPolicyNever.
	vbox packStart: commentSw expand: false fill: true padding: 0.
	self showComments ifTrue: [ commentSw show ].

	(commentLbl := GtkLabel new: '') 
		setName: 'commentLbl';
		show.
	commentSw addWithViewport: commentLbl.
	commentSw getChild setShadowType: Gtk gtkShadowNone.

	self hookupPopupMenuTo: ebox.
!

attachViews
	childViews do: 
		[ :each | 
		(each = tree) ifFalse:
			[vbox 
				packStart: each external 
				expand: false 
				fill: false
				padding: 0. ]. ].
		
	vbox packEnd: tree external expand: true fill: true padding: 0.
	"For some reason, doing these in the opposite order stops the popup
	 menu from appearing."
    self hookupPopupMenuTo: tree treeView.
	tree treeView setName: 'methodTree'.
!
!

ClassView methodsFor: 'change update'!

classChanged
	| cl joinStrings |
	popupMenuItems := nil.
	cl := self data displayClass.
	superLblBtn getChild setMarkup: 
		'<i>', cl superclass printString, '</i>'.
	superLblBtn setSensitive: cl superclass notNil.
	classLbl setMarkup: 
		'<big>', cl printString, '</big>'.
	cl comment 
		ifNil: [ commentLbl hide ]
		ifNotNil: 
			[ :c | commentLbl setText: c; show. ].
!
!

ClassView methodsFor: 'delegation'!

on: aEvent send: aSelector to: aReceiver with: aData
	treeMsgs at: aEvent ifPresent: 
		[ :ev | ^tree on: ev send: aSelector to: aReceiver with: aData ].
	super on: aEvent send: aSelector to: aReceiver with: aData.
!

on: aEvent send: aSelector to: aReceiver
	treeMsgs at: aEvent ifPresent: 
		[ :ev | ^tree on: ev send: aSelector to: aReceiver ].
	super on: aEvent send: aSelector to: aReceiver.
!

on: aEvent update: aReceiver with: aBlock
	treeMsgs at: aEvent ifPresent: 
		[ :ev | ^tree on: ev update: aReceiver with: aBlock ].
	super on: aEvent update: aReceiver with: aBlock.
!

on: aEvent updateWith: aBlock
	treeMsgs at: aEvent ifPresent: 
		[ :ev | ^tree on: ev updateWith: aBlock ].
	super on: aEvent updateWith: aBlock.
!
!

ClassView methodsFor: 'popup menus'!

contextMenuItems
    contextMenuItems isNil ifTrue: 
		[ contextMenuItems := LookupTable new.
		(contextMenuItems at: 'classEbox' put: MenuItemList new)
			section: 'Viewing' 
			add: 'Show comments' 
			sending: #toggleShowComments 
			to: self. 
		(contextMenuItems at: 'methodTree' put: MenuItemList new)
			section: 'Editing' add: 'New method...' sending: #newMethodCallback to: self;
			section: 'Browsing' add: 'Browse method...' sending: #browseMethodCallback to: self. ].
    ^contextMenuItems.
!

addPopupMenuItems: aMenuCollector forWidget: aWidget
	self contextMenuItems 
		at: 
			(aWidget ifNil: [ 'default' ] ifNotNil: [ :w | w getName ])
		ifPresent: 
			[ :mil | aMenuCollector addAll: mil. ].
    ^aMenuCollector.
!
!

ClassView methodsFor: 'accessing'!

external
	^ebox
!

showComments: aBoolean
	showComments := aBoolean.
	aBoolean 
		ifTrue: [ commentSw show ]
		ifFalse: [ commentSw hide ].
!

showComments	
	showComments isNil ifTrue: 
		[ ShowComments isNil ifTrue: 
			[ ShowComments := true ].
		showComments := ShowComments ].
	^showComments.
!

toggleShowComments
	self showComments: self showComments not.
!
!

ClassView methodsFor: 'events'!

superBtnCallback
	(events named: #superBtnClicked) activate
!

newMethodCallback
    | s cat cls |
    s := tree selection ifNil: [ ^self ].
	cat := s category ifNil: [ 'uncategorized' ].
	cls := s isClassNode 
	    ifTrue: [ data displayClass class ]
	    ifFalse: [ data displayClass ].
	(events named: #newMethodClicked) activateWith: cls -> cat.
!

browseMethodCallback
	tree rowActivatedCallback: self.		
!
!
