"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

GuiData subclass: #EvaluatorData
	instanceVariableNames: 'target storedText'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'!

WindowedView subclass: #EvaluatorView
	instanceVariableNames: 'frame text textView keyBindings dbg'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!

!EvaluatorData methodsFor: 'change update'!

target: aObject
	target := aObject.
	self changeState: #targetChanged.
!

target
	^target
!

storeText: aString
	storedText := aString.
!

storedText
	^storedText
!
!

!EvaluatorData methodsFor: 'evaluating'!

evaluate: aString sendResultTo: aBlock
	| r |
	r := Behavior 
		evaluate: aString
		to: target
		ifError: [ :fname :line :msg | Transcript print: msg. ^self ].
    
	aBlock value: r.
!
!

!EvaluatorView methodsFor: 'initialization'!

initialize
	self events add: #dirtyChanged.
	super initialize.
!

buildWidgets
    | scroll pol |

    super buildWidgets.

	(frame := GtkFrame new: nil) show.
 
    text := GtkTextBuffer new: nil.

	(scroll := GtkScrolledWindow new: nil vadjustment: nil) show.
    frame add: scroll.
    pol := Gtk gtkPolicyNever.
    scroll setPolicy: pol vscrollbarPolicy: pol.
	
	(textView := GtkTextView new) show.
    scroll add: textView.

    textView setBuffer: text.

    text on: 'insert-text' send: #changedCallback to: self.
    text on: 'delete-range' send: #changedCallback to: self.
	textView on: 'key-release-event' send: #keyReleased:key: to: self.
	
	self hookupPopupMenuTo: textView.
!

external
	^frame
!

buildPopupMenuItems
	super buildPopupMenuItems.
	popupMenuItems
		section: 'Editing' add: 'Cut (ctrl-X)' sending: #cutCallback to: self;
		section: 'Editing' add: 'Copy (ctrl-C)' sending: #copyCallback to: self;
		section: 'Editing' add: 'Paste (ctrl-V)' sending: #pasteCallback to: self;
		section: 'Evaluating' add: 'Do it (ctrl-D)' sending: #doItCallback to: self;
		section: 'Evaluating' add: 'Print it (ctrl-P)' sending: #printItCallback to: self;
		section: 'Evaluating' add: 'Get it... (ctrl-G)' sending: #getItCallback to: self;
		section: 'Evaluating' add: 'Inspect it... (ctrl-I)' sending: #inspectItCallback to: self;
		section: 'Browsing' add: 'Senders... (alt-S)' sending: #sendersCallback to: self;
		section: 'Browsing' add: 'Implementors... (alt-I)' sending: #implementorsCallback to: self.
!

setup: aData stateChange: aStateChange
	self 
		data: aData;
		stateChange: aStateChange updateWith: [ self targetChanged ].
!

targetChanged
	data storedText ifNotNil: 
		[ :s | text setText: s len: -1 ].
!
!

!EvaluatorView methodsFor: 'accessing'!

external
	^frame
!

text
	^text
!

textView
	^textView
!

scroll
	^textView getParent 
!
!

!EvaluatorView methodsFor: 'events'!

defaultClipboard
	"GDK_SELECTION_CLIPBOARD = 69 as at Gtk 2.2.1; GDK_NONE = 0, but is 
	 supported for backwards compatibility. Ideally the install should 
	 provide these consts from the Gtk source."
	^GtkClipboard get: 69.
!

cutCallback
	text cutClipboard: self defaultClipboard defaultEditable: true.
!

copyCallback
	text copyClipboard: self defaultClipboard.
!

pasteCallback
	text 
		pasteClipboard: self defaultClipboard
		overrideLocation: nil
		defaultEditable: true.
!

clearText
	text clearText.
!

setText: aString
	text setText: aString len: -1.
	self dirty: false.
!

getText
	^text getText.
!

dirty: aBoolean
	"Changed this to trigger every time, although, once it's been 
	debugged, it need only fire on a state change, I think."
	"FileStream stdout print: 'dirtyChanged'; flush."
	dirty ~= aBoolean ifTrue: 
		[ super dirty: aBoolean.
		(events named: #dirtyChanged) activate ].
!

changedCallback
	self dirty: true.
!

discardCallback
	self dirty: false.
!

keyBindings
	keyBindings isNil ifTrue: 
		[ keyBindings := Dictionary new.
		keyBindings 
			at: (Gdk gdkControlMask -> 16r064) put: #doItCallback "ctrl-d";
			at: (Gdk gdkControlMask -> 16r067) put: #getItCallback "ctrl-g";
			at: (Gdk gdkControlMask -> 16r069) put: #inspectItCallback "ctrl-i";
			at: (Gdk gdkControlMask -> 16r070) put: #printItCallback "ctrl-p";
			at: (Gdk gdkMod1Mask -> 16r069) put: #implementorsCallback "alt-i";
			at: (Gdk gdkMod1Mask -> 16r073) put: #sendersCallback "alt-s". ].
	^keyBindings.
		
!

keyReleased: aSender key: aGdkEvent
	aGdkEvent class = GdkEvent ifTrue: 
		[ "This does seem to be necessary" 
		aGdkEvent changeClassTo: GdkEventKey. ].
	(aGdkEvent isKindOf: GdkEventKey) 
		ifFalse: [ self error: 'keyReleased was not a key event! (', aGdkEvent class name, ')' ].

	"FIXME I'm pretty sure this is not the correct way to handle key bindings."
	self keyBindings 
		at: ((aGdkEvent state value bitAnd: (Gdk gdkMod1Mask bitOr: Gdk gdkControlMask))
			-> aGdkEvent keyval value) 
		ifPresent: [ :sym | ^self perform: sym ].
	^false
!

evaluateSendResultTo: aBlock
	data evaluate: self selectionOrText sendResultTo: aBlock.
!

evaluate
	self evaluateSendResultTo: [ :r | "Discard result" ].
!

doItCallback
    self evaluate
!

printItCallback
	| start end |
	self evaluateSendResultTo: 
		[ :r | | s |
		start := GtkTextIter type new.
		end := GtkTextIter type new.
		text 
			getSelectionBounds: start end: end;
			placeCursor: end.
		s := r printString.
		text 
			insertInteractiveAtCursor: s
			len: s size
			defaultEditable: true. ].
!

getItCallback
	self evaluateSendResultTo: 
		[ :r | (WindowFactory buildWorkspaceWindow: r) show.
		Primitive updateViews ].
!

inspectItCallback
	self evaluateSendResultTo: 
		[ :r | (WindowFactory buildInspectorWindow: r) show.
		Primitive updateViews ].
!

selectionOrLine
	| r |
	r := text getSelectionText.
	r notEmpty ifTrue: [ ^r ].
	^text getCurrentLine.
!

selectionOrWord
	| r |
	r := text getSelectionText.
	r notEmpty ifTrue: [ ^r ].
	^text getCurrentWord.
!

selectionOrText
	| r |
	r := text getSelectionText.
	r notEmpty ifTrue: [ ^r ].
	^text getText.
!

selectionOrWordAsSymbol
	| r |
	r := self selectionOrWord.
	r notEmpty 
		ifTrue: [ ^r asSymbol ]
		ifFalse: [ ^nil ].
!

sendersCallback
	"FIXME - Parse text to try to identify multi-word selectors"
	self selectionOrWordAsSymbol ifNotNil: 
		[ :sym |
		(WindowFactory buildMethodSetWindow: 
			(MethodList forSendersOf: sym)) show.
		Primitive updateViews. ].
!

implementorsCallback
	"FIXME - Parse text to try to identify multi-word selectors"
	self selectionOrWordAsSymbol ifNotNil: 
		[ :sym |
		(WindowFactory buildMethodSetWindow: 
			(MethodList forImplementorsOf: sym)) show.
		Primitive updateViews. ].
!
!
