"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"


EvaluatorData subclass: #InspectorData
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'!
!
!

WindowedView subclass: #InspectorView
	instanceVariableNames: 'ebox vbox hbox tree infoTxt eval pmnuItems treeMenuItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!

InspectorData methodsFor: 'change update'!

getNodes: aNode
	| r instVars |
	aNode notNil ifTrue: [ ^nil ].
	r := OrderedCollection new.
	r add: ('self (', target class article, ' ', target class displayString, ')')
		-> target.

	instVars := target class allInstVarNames.

	1 to: instVars size do:
		[ :i | r add: (instVars at: i) -> (target instVarAt: i) ].

	1 to: target validSize do:
		[ :i | | each |
		each := target basicAt: i.
		r add: (i displayString, ': ', each displayString) -> each ].

	"There is a better way, I'm sure"
	"(target isKindOf: Dictionary) ifTrue: 
		[ | i |
		i := 1.
		target keysAndValuesDo: 
			[ :k :v | | a |
			a := k -> v.
			r add: (i printString, ': ', a printString) -> a.
			i := i + 1. ]."
	^r
!

getCols: aNode
	^Array with: (aNode ifNil: [ '' ] ifNotNil: [ :n | n key ]).
!
!

InspectorView methodsFor: 'initialization'!

external
	^ebox
!

buildSubsidiaryViews
	self add: (tree := PTree new).
	tree
		buildColumns: #('');
		nodesMsg: #getNodes: colsMsg: #getCols:;
		on: #nodeSelected send: #nodeSelected to: self;
		on: #nodeActivated send: #nodeActivated to: self;
		sorted: false.
	self add: (eval := EvaluatorView new).
!

buildWidgets
	| scroll frame textView pol |
	super buildWidgets.
    (ebox := GtkEventBox new) show.

	"General layout"
	
	(vbox := GtkVBox new: false spacing: 0) show.
	ebox add: vbox.
	
	(hbox := GtkHBox new: false spacing: 0) show.
	vbox packStart: hbox expand: true fill: true padding: 0.
	
	"Info text"

    infoTxt := GtkTextBuffer new: nil.

	(frame := GtkFrame new: nil) show.
    hbox packEnd: frame expand: true fill: true padding: 0.
	(scroll := GtkScrolledWindow new: nil vadjustment: nil) show.
    frame add: scroll.
    pol := Gtk gtkPolicyAutomatic.
    scroll setPolicy: pol vscrollbarPolicy: pol.
	(textView := GtkTextView new) show.
    scroll add: textView.

    textView setBuffer: infoTxt.
!

attachViews
	hbox packStart: tree external expand: true fill: true padding: 0.
	vbox packEnd: eval external expand: true fill: true padding: 0.
    self hookupPopupMenuTo: tree treeView.
!

setup: aData stateChange: aStateChange
	self 
		data: aData;
		stateChange: aStateChange updateWith: [ self clearTextPanes. ].
	tree 
		data: aData;
		rebuildStateChange: aStateChange.
	eval
		setup: aData
		stateChange: aStateChange.
!
!

InspectorView methodsFor: 'popup menus'!

treeMenuItems
    treeMenuItems isNil ifTrue: 
		[ treeMenuItems := MenuItemList new.
		treeMenuItems 
			section: 'Browsing' add: 'Inspect...' sending: #inspectSelectedCallback to: self;
			section: 'Browsing' add: 'Browse class...' sending: #browseSelectedClassCallback to: self ].
    ^treeMenuItems.
!

addPopupMenuItems: aMenuItemList forWidget: aWidget
    (aWidget = tree treeView)
		ifTrue: [ aMenuItemList addAll: self treeMenuItems. ].
    ^aMenuItemList.
!
!

InspectorView methodsFor: 'events'!

browseSelectedClassCallback
	(WindowFactory buildClassWindow: tree selection value class) show.
	Primitive updateViews.
!

inspectSelectedCallback
	(WindowFactory buildInspectorWindow: tree selection value) show.
	Primitive updateViews.
!

clearTextPanes
	infoTxt clearText.
	eval clearText.
!

nodeSelected
	infoTxt setText: tree selection value printString len: -1.
!

nodeActivated
	(WindowFactory buildInspectorWindow: tree selection value) show.
	Primitive updateViews.
!
!
