"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

TreeData subclass: #MethodList
	instanceVariableNames: 'description items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'
!

WindowedView subclass: #MethodSetView
	instanceVariableNames: 'ebox vbox tree lbl treeMenuItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!


MethodList class methodsFor: 'common lists'!

sendersOf: aSymbol
    | col |
    col := OrderedCollection new.
    Class allSubclassesDo: 
	    [:meta | 
	    (meta whichSelectorsReferTo: aSymbol) 
			do: [:sel | col add: meta >> sel].
	    (meta instanceClass whichSelectorsReferTo: aSymbol) 
			do: [:sel | col add: meta instanceClass >> sel]].
	^col
!

implementorsOf: aSymbol
    | col |
    col := OrderedCollection new.
    Class allSubclassesDo: 
	    [:meta | 
	    (meta includesSelector: aSymbol) 
			ifTrue: [col add: meta >> aSymbol].
	    (meta instanceClass includesSelector: aSymbol) 
			ifTrue: [col add: meta instanceClass >> aSymbol]].
	^col
!
!

MethodList class methodsFor: 'instance creation'!

forSendersOf: aSymbol
	^(self new) 
		description: 'Senders of ', aSymbol printString;
		items: (self sendersOf: aSymbol); 
		yourself.
!

forImplementorsOf: aSymbol
	^(self new) 
		description: 'Implementors of ', aSymbol printString;
		items: (self implementorsOf: aSymbol); yourself.
!
!

MethodList methodsFor: 'intialization'!

items: aItems
	items := Array withAll: aItems.
    self changeState: #listChanged.
!

description: aDescription
	description := aDescription.
    self changeState: #descriptionChanged.
!
!

MethodList methodsFor: 'accessing'!

description
	^description
!

getCols: aCompiledMethod
    ^Array with: aCompiledMethod printString
!

getNodes: aNode
	aNode notNil ifTrue: [ ^nil ].
	^items.
!
!

MethodSetView methodsFor: 'initialization'!

buildSubsidiaryViews
	self add: (tree := PTree new).
	tree
		buildColumns: #('Method');
		nodesMsg: #getNodes: colsMsg: #getCols:.
	tree on: #nodeActivated updateWith: [ self methodActivated ].
!

buildWidgets
	super buildWidgets.
    (ebox := GtkEventBox new) show.
	(vbox := GtkVBox new: false spacing: 0) show.
	ebox add: vbox.
	(lbl := GtkLabel new: '') show.
	vbox packStart: lbl expand: false fill: true padding: 12.
	self hookupPopupMenuTo: ebox.
!

attachViews
	vbox packEnd: tree external expand: true fill: true padding: 0.
!

descriptionChanged
	lbl setText: data description.
!

initialize
	super initialize.
!

setup: aData listChange: aListSC descChange: aDescSC
	self data: aData.
	self 
		stateChange: aDescSC
		updateWith: [ self descriptionChanged ].

	"Core tree messages"
	tree
		data: aData;
		nodesMsg: #getNodes:;
		colsMsg: #getCols:;
		stateChange: aListSC
			updateWith: [ tree rebuildTree ].
!
!

MethodSetView methodsFor: 'accessing'!

external
	^ebox
!

title
	^data description.
!
!

!MethodSetView methodsFor: 'popup menus'!

treeMenuItems
    treeMenuItems isNil ifTrue: 
		[ treeMenuItems := MenuItemList new.
		treeMenuItems 
			section: 'Browsing' add: 'Browse method...' sending: #browseMethodCallback to: self. ].
    ^treeMenuItems.
!

addPopupMenuItems: aMenuItemList forWidget: aWidget
    (aWidget = tree treeView)
		ifTrue: [ aMenuItemList addAll: self treeMenuItems. ].
    ^aMenuItemList.
!
!

!MethodSetView methodsFor: 'events'!

methodActivated
	tree selection ifNotNil: 
		[ :cm |
		(WindowFactory buildMethodWindow: tree selection) show.
		Primitive updateViews ].
!
!
