"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

TreeData subclass: #MethodData
	instanceVariableNames: 'mth cls cat'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'
!

WindowedView subclass: #MethodView
	instanceVariableNames: 'ebox vbox classBtn methodLbl acceptBtn cancelBtn text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!

MethodData methodsFor: 'accessing'!

displayMethod: aCompiledMethod class: aClass category: aCategory
	mth := aCompiledMethod.
	cls := aClass.
	cat := aCategory.
	self changeState: #methodChanged.
	Primitive updateViews.
!

displayMethod
	^mth
!

methodClass
	^cls
!

methodSelector
	^mth 
		ifNotNil: [ :m | m selector ]
		ifNil: [ '(new method)' ].
!

methodCategory
	^cat
!

methodSource
	^mth 
		ifNotNil: [ :m | m methodSourceString ] 
		ifNil: [ '"New method"' ].
!

description
	mth ifNotNil: [ :mc | ^mc displayString ].
	^cls displayString, ' >> (new method)'. 
!
!

MethodData methodsFor: 'compiling'!

compile: aString
    | cm newSel |

    cm := cls
		compile: aString
		classified: cat
		ifError: 
			[ :fname :lineNo :errorString | 
			(lineNo printString, ': ', errorString) printNl.
			^false. ].

    "Retrieve selector"
	self displayMethod: cm class: cls category: cat.

    "Change Log entry"
    "CurrentChanges methodChange: cls -> sel."

    ^true
!
!

MethodView methodsFor: 'initialization'!

methodChange: aStateChange
	self stateChange: aStateChange updateWith: [ self methodChanged ].
!

resetText
	text setText: data methodSource.
	text dirty: false.
!

methodChanged
	| mth joinStrings |
	popupMenuItems := nil.
	
	classBtn getChild setMarkup: 
		'<big>', (Pango quoteMarkup: data methodClass displayString), '</big>'.
	methodLbl setMarkup: 
		'<big>', (Pango quoteMarkup: data methodSelector displayString), '</big>'.

	self resetText.
!

initialize
	self events add: #classBtnClicked.
	super initialize.
!

buildSubsidiaryViews
	text := EvaluatorView new.
	text on: #dirtyChanged send: #textDirtyChanged to: self.
	"This style of event handler seems to unhook itself somehow."
	"text on: #dirtyChanged updateWith: [ self textDirtyChanged ]."
!

buildWidgets
	| al hb1 hb2 lbl sw |
	super buildWidgets.
	
	(ebox := GtkEventBox new) show.
	
	(vbox := GtkVBox new: false spacing: 3) show.
	ebox add: vbox.
	
	(hb1 := GtkHBox new: false spacing: 3) show.
	vbox packStart: hb1 expand: false fill: false padding: 0.

	(al := GtkAlignment new: 0.5 yalign: 0.5 xscale: 0.0 yscale: 0.0) show.
	hb1 add: al.
	
	(hb2 := GtkHBox new: false spacing: 3) show.
	al add: hb2.
	
	(classBtn := GtkButton newWithLabel: '')
		setRelief: Gtk gtkReliefNone;
		on: 'clicked' send: #classBtnCallback to: self;
		show.
	hb2 packStart: classBtn expand: false fill: false padding: 0.
	
	(lbl := GtkLabel new: '>>') show.
	hb2 packStart: lbl expand: false fill: false padding: 0.

	(methodLbl := GtkLabel new: '') show.
	hb2 packStart: methodLbl expand: false fill: false padding: 0.

	(cancelBtn := GtkButton new) show.
	cancelBtn 
		add: 
			((GtkImage newFromStock: Gtk gtkStockCancel size: Gtk gtkIconSizeButton) 
				show; 
				yourself);
		setSensitive: false;
		on: 'clicked' send: #cancelClick to: self.
	hb1 packEnd: cancelBtn expand: false fill: false padding: 0.
	
	(acceptBtn := GtkButton new) show.
	acceptBtn 
		add: 
			((GtkImage newFromStock: Gtk gtkStockApply size: Gtk gtkIconSizeButton) 
				show; 
				yourself);
		setSensitive: false;
		on: 'clicked' send: #acceptClick to: self.
	hb1 packEnd: acceptBtn expand: false fill: false padding: 0.

	self hookupPopupMenuTo: ebox.
!

attachViews
	vbox packEnd: text external expand: true fill: true padding: 0.
!
!

MethodView methodsFor: 'events'!

textDirtyChanged
	"('text isDirty = ', text isDirty printString) printNl."
	(Array with: acceptBtn with: cancelBtn) do: 
		[ :btn | btn setSensitive: text isDirty].
!

classBtnCallback
	(events named: #classBtnClicked) activate.
!

acceptClick
    (data compile: text getText) ifTrue: 
		[ text dirty: false. ].    
!

cancelClick
    self resetText.
!
!

MethodView methodsFor: 'accessing'!

external
	^ebox
!
!
