"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

TreeData subclass: #NamespaceData
	instanceVariableNames: 'displayNamespace listMode ancestors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'
!

WindowedView subclass: #NamespaceView
	instanceVariableNames: 'vbox ebox lbl tree treeMenuItems treeEvents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!

NamespaceData methodsFor: 'accessing'!

ancestors
    | todo |
    ancestors isNil ifTrue: 
		[ todo := Array streamContents: 
			[ :stream |
			self displayNamespace allClassesDo: 
				[ :each | stream nextPut: each ] ].
		ancestors := todo select: 
			[ :each |
			each superclass isNil or: 
				[ each superclass environment 
					~= displayNamespace ] ]. ].
    ^ancestors.
!

displayNamespace
    displayNamespace isNil ifTrue: [ displayNamespace := Smalltalk ].
    ^displayNamespace.!

displayNamespace: aNamespace
    displayNamespace := aNamespace.
    self changeState: #namespaceChanged.
!
!

NamespaceData methodsFor: 'change update'!

getCols: aClass
    ^Array with: (aClass name ifNil: [ '' ])
!

getNodes: aNode
    ^(self listMode 
	  ifTrue: [ self getNodesForList: aNode ]
	  ifFalse: [ self getNodesForTree: aNode ])
!

getNodesForList: aNode
    | cls |
    aNode notNil ifTrue: [ ^nil ].
    cls := Array streamContents: [:stream | 
	self displayNamespace allClassesDo: [:each | 
	    stream nextPut: each
	]
    ].
    cls := cls asSortedCollection: 
	[ :a :b | 
	    [ (a name) <= (b name) ] on: Error do: 
		[ :sig | a print. ' ' print. b printNl. sig error: 'rethrown' ]
	].
    ^cls.
!

getNodesForTree: aNode
    aNode isNil ifTrue: [ ^self ancestors ].
    aNode isClass ifTrue: 
		[ ^aNode subclasses select: 
			[ :each | each environment = displayNamespace ] ].
    ^nil.
!

listMode
    listMode isNil ifTrue: [ listMode := false ].
    ^listMode
!
!

NamespaceData methodsFor: 'events'!

createSubclassOf: aClass named: aName
    | ns |
    "FIXME - Add to ChangeLog"
    ns := Namespace current.
    Namespace current: self data displayNamespace.
    aClass 
		subclass: aName asSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: aClass category.
    Namespace current: ns.
    self changeState: #namespaceChanged.
!
!

NamespaceView methodsFor: 'initialization'!

buildSubsidiaryViews
	self add: (tree := PTree new).
	tree
		buildColumns: #('Class');
		nodesMsg: #getNodes: colsMsg: #getCols:.
!

namespaceChanged
	popupMenuItems := nil.
	lbl setMarkup: 
		'<big>', self data displayNamespace printString, '</big>'.
!

initialize
	super initialize.
	(treeEvents := LookupTable new)
		at: #classSelected put: #nodeSelected;
		at: #classActivated put: #nodeActivated.
!

setup: aData stateChange: aStateChange nodesMsg: aNodesMsg colsMsg: aColsMsg
	self data: aData.
	self 
		stateChange: aStateChange
		updateWith: [ self namespaceChanged ].

	"Core tree messages"
	tree
		data: aData;
		nodesMsg: aNodesMsg;
		colsMsg: aColsMsg;
		stateChange: aStateChange
			updateWith: [ tree rebuildTree ].
!

on: aEvent send: aSelector to: aReceiver
	treeEvents at: aEvent ifPresent: 
		[ :ev | ^tree on: ev send: aSelector to: aReceiver ].
!

on: aEvent update: aReceiver with: aBlock
	treeEvents at: aEvent ifPresent: 
		[ :ev | ^tree on: ev update: aReceiver with: aBlock ].
!

on: aEvent updateWith: aBlock
	treeEvents at: aEvent ifPresent: 
		[ :ev | ^tree on: ev updateWith: aBlock ].
!

buildWidgets
	super buildWidgets.
    (ebox := GtkEventBox new) show.
	(vbox := GtkVBox new: false spacing: 0) show.
	ebox add: vbox.
	(lbl := GtkLabel new: '') show.
	vbox packStart: lbl expand: false fill: true padding: 12.
	self hookupPopupMenuTo: ebox.
!

attachViews
	vbox packEnd: tree external expand: true fill: true padding: 0.
    self hookupPopupMenuTo: tree treeView.
!
!

NamespaceView methodsFor: 'accessing'!

external
	^ebox
!
!

!NamespaceView methodsFor: 'events'!

fileOutClass: aClass to: aFileName
    | fs |
    fs := FileStream open: aFileName mode: FileStream write.
    
    aClass fileOutOn: fs.
    
    fs close.    
!

browseClassCallback
	tree rowActivatedCallback: self.	
!

promptFileOutClass
    | cls pt fn fs |
    cls := tree selection.
    cls isNil ifTrue: 
		[pt := Prompter 
			title: 'No class is selected' 
			entry: nil
			buttons: (Prompter buttonsOk).
		pt run.
		pt destroy.
		^self ].

    fn := 'changes/', (cls name ifNil: [ 'Unknown' ]), '.st'.

    pt := Prompter 
		title: ('Filing out ', cls name ifNil: [ 'Unknown' ]) 
		entry: 'File name: '
		buttons: (Prompter buttonsOkCancel).
    pt entryText: fn.
    (pt run = Gtk gtkResponseOk) 
		ifTrue: 
			[ self fileOutClass: cls to: (self makeFileName: pt entryText) ].
    pt destroy.
!

promptFindClass
	Transcript print: 'Find Class is not yet implemented'.
!

makeFileName: aFileName
    "Strip out spaces, remove non-printing chars, etc."
    ^aFileName
!

promptSubclass
    | cls pt |
    cls := tree selection.
    pt := Prompter 
		title: ('Create subclass of: ', cls name ifNil: [ 'Unknown' ]) 
		entry: 'Name: '
		buttons: (Prompter buttonsOkCancel).
	[
		(pt run = Gtk gtkResponseOk) 
			ifTrue: [ | nm |
				nm := pt entryText.
				data createSubclassOf: cls named: nm.
				(tree find: [ :cls | cls name = nm ]) ifNotNil: 
					[ :it | tree selectIter: it ].
			].
	] ensure: 
		[ pt destroy ].
!
!

!NamespaceView methodsFor: 'popup menus'!

treeMenuItems
    treeMenuItems isNil ifTrue: 
		[ treeMenuItems := MenuItemList new.
		treeMenuItems 
			section: 'Browsing' add: 'Refresh' sending: #namespaceChanged to: self;
			section: 'Browsing' add: 'Browse class...' sending: #browseClassCallback to: self;
			section: 'Filing' add: 'File out class...' sending: #promptFileOutClass to: self;
			section: 'Editing' add: 'Subclass...' sending: #promptSubclass to: self. ].
    ^treeMenuItems.
!

addPopupMenuItems: aMenuItemList forWidget: aWidget
    (aWidget = tree treeView)
		ifTrue: [ aMenuItemList addAll: self treeMenuItems. ].
    ^aMenuItemList.
!

buildPopupMenuItems
    super buildPopupMenuItems.
    popupMenuItems
		section: 'Searching'
		add: 'Find class...'
		sending: #promptFindClass 
		to: data displayNamespace.
!
!
