"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

TreeData subclass: #NamespacesData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'
!

WindowedView subclass: #NamespacesView
	instanceVariableNames: 'vbox ebox tree treeMenuItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!

NamespacesData methodsFor: 'change update'!

getCols: aClass
    ^Array with: (aClass name ifNil: [ '' ])
!

getNodes: aNode
    aNode isNil ifTrue: [ ^Array with: Smalltalk ].
    aNode isNamespace ifTrue: [ ^aNode subspaces ].
    ^nil.
!
!

NamespacesView methodsFor: 'initialization'!

buildSubsidiaryViews
	| d |
	self add: (tree := PTree new).
	tree buildColumns: #('Namespace').
!

setup: aData stateChange: aStateChange nodesMsg: aNodesMsg colsMsg: aColsMsg
	self data: aData.
	"self 
		stateChange: aStateChange
		updateWith: [ self namespacesChanged ]."
		
	tree
		data: aData;
		nodesMsg: #getNodes:;
		colsMsg: #getCols:;
		stateChange: aStateChange
			updateWith: [ tree rebuildTree ].
!

on: aEvent send: aSelector to: aReceiver
	tree on: (self treeEventName: aEvent) send: aSelector to: aReceiver.
!

on: aEvent update: aReceiver with: aBlock
	tree on: (self treeEventName: aEvent) update: aReceiver with: aBlock.
!

on: aEvent updateWith: aBlock
	tree on: (self treeEventName: aEvent) updateWith: aBlock.
!

buildWidgets
	| lbl |
	super buildWidgets.
    (ebox := GtkEventBox new) show.
	(vbox := GtkVBox new: false spacing: 0) show.
	ebox add: vbox.
	(lbl := GtkLabel new: '')
		setMarkup: '<big>Namespaces</big>';
		show.
	vbox packStart: lbl expand: false fill: true padding: 12.
	self hookupPopupMenuTo: ebox.
!

attachViews
	vbox packEnd: tree external expand: true fill: true padding: 0.
	self hookupPopupMenuTo: tree treeView.
!
!

NamespacesView methodsFor: 'accessing'!

external
	^ebox
!

selection
	^tree selection
!
!

NamespacesView methodsFor: 'private'!

treeEventName: aEvent
	(aEvent startsWith: #namespace) ifFalse: [ self error: 'No such event ', aEvent printString ].
	^(aEvent copyReplaceFrom: 1 to: 9 with: 'node') asSymbol.
!

rebuildTree
	tree rebuildTree.
!
!

!NamespacesView methodsFor: 'popup menus'!

treeMenuItems
    treeMenuItems isNil ifTrue: 
		[ treeMenuItems := MenuItemList new.
		treeMenuItems 
			section: 'Browsing' add: 'Browse Namespace...' sending: #browseNSCallback to: self;
			section: 'Viewing' add: 'Refresh' sending: #rebuildTree to: self. ].
    ^treeMenuItems.
!

addPopupMenuItems: aMenuItemList forWidget: aWidget
    (aWidget = tree treeView)
		ifTrue: [ aMenuItemList addAll: self treeMenuItems. ].
    ^aMenuItemList.
!

!NamespacesView methodsFor: 'events'!

browseNSCallback
	tree rowActivatedCallback: self.
!
!


