"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

TreeData subclass: #StacktraceData
	instanceVariableNames: 'errorMsg process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'
!

WindowedView subclass: #NotifierView
	instanceVariableNames: 'vbox ebox lbl tree treeMenuItems'
	classVariableNames: 'handleErrors'
	poolDictionaries: ''
	category: 'Mumble-Main'!

StacktraceData methodsFor: 'accessing'!

error: aError process: aProcess
    errorMsg := aError.
	process := aProcess.
    self changeState: #processChanged.
!

errorMsg
	^errorMsg
!

resumeProcess
	process resume.
!

terminateProcess
	"This doesn't seem to terminate the process very effectively, though."
	process terminate.
!
!

StacktraceData methodsFor: 'change update'!

getCols: aContext
    ^Array with: aContext printString
!

getNodes: aNode
    | contexts context lastContext suspendedContext stacktrace |
    aNode notNil ifTrue: [ ^nil ].

	context := process suspendedContext.
    lastContext := context environment.
    stacktrace := OrderedCollection new.
    contexts := OrderedCollection new.

    "Skip top contexts that are internal to the exception-handling
     system."

    [ context ~~ lastContext 
		and: 
			[ context isInternalExceptionHandlingContext ] ] 
		whileTrue: [ context := context parentContext. ].

    [ context == lastContext ] 
		whileFalse: 
			[ context isDisabled ifFalse: 
				[ stacktrace add: context. ].
			context := context parentContext. ].
			
	^stacktrace
!
!

NotifierView methodsFor: 'initialization'!

buildSubsidiaryViews
	self add: (tree := PTree new).
	tree
		buildColumns: #('Context');
		nodesMsg: #getNodes: colsMsg: #getCols:;
		sorted: false.
!

dataChanged
	popupMenuItems := nil.
	lbl setMarkup: 
		'<big>', (Pango quoteMarkup: self data errorMsg), '</big>'.
!

setup: aData stateChange: aStateChange
	self data: aData.
	self 
		stateChange: aStateChange
		updateWith: [ self dataChanged ].

	"Core tree messages"
	tree
		data: aData;
		nodesMsg: #getNodes:;
		colsMsg: #getCols:;
		stateChange: aStateChange
			updateWith: [ tree rebuildTree ].
!

buildWidgets
	| hbox btn |
	super buildWidgets.
    (ebox := GtkEventBox new) show.
	
	(vbox := GtkVBox new: false spacing: 0) show.
	ebox add: vbox.
	
	(hbox := GtkHBox new: false spacing: 0) show.
	vbox packStart: hbox expand: false fill: true padding: 12.
	
	(lbl := GtkLabel new: '') show.
	hbox packStart: lbl expand: false fill: true padding: 6.
	
	(btn := GtkButton newWithLabel: 'Abandon') show.
	hbox packEnd: btn expand: false fill: true padding: 0.
	btn on: 'clicked' send: #abandonClicked to: self.

	(btn := GtkButton newWithLabel: 'Proceed') show.
	hbox packEnd: btn expand: false fill: true padding: 0.
	btn on: 'clicked' send: #proceedClicked to: self.
	
	self hookupPopupMenuTo: ebox.
!

attachViews
	vbox packEnd: tree external expand: true fill: true padding: 0.
    self hookupPopupMenuTo: tree treeView.
!
!

NotifierView methodsFor: 'accessing'!

external
	^ebox
!
!

NotifierView methodsFor: 'events'!

proceedClicked
	data resumeProcess.
	self close.
!

abandonClicked
	data terminateProcess.
	self close.
!
!


!NotifierView class methodsFor: 'initialization'!

initialize
	handleErrors := false.
!
!

!NotifierView class methodsFor: 'exception handling'!

debuggerClass
    ^nil
!

debuggingPriority
    ^1
!

handleErrors
	handleErrors isNil ifTrue: [ handleErrors := false ].
	^handleErrors
!

handleErrors: aBoolean
	handleErrors := aBoolean
!

openOn: process message: message
    handleErrors := false.
    [ | sd nv |
        [ process isSuspended ] whileFalse: [ Processor yield ].

		"If we've just suspended the event loop, we'd better start 
		 another, otherwise we won't see the window!"
		
		"This test doesn't work because the process is a call-in process,
			 not the original one (which is presumably blocking).
		 (Gtk activeDispatcherIs: process)"
		"FIXME This is a hack"
		(process name notNil and: [ process name match: 'call-in process' ])
			ifTrue: [ Gtk newDispatcher ].

		Processor activeProcess name: 'Notifier/Debugger'.

		sd := StacktraceData new.		
        (nv := self new)
			setup: sd stateChange: #processChanged.
 	    sd error: message process: process.		
		(WindowFactory 
			buildWindowFor: nv 
			title: (WindowView safeTitleString: message)) show.
		
		Primitive updateViews.

        handleErrors := true.
    ] fork.

    process suspend.
!
!
