"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

Dialog subclass: #Prompter
	instanceVariableNames: ''
	classVariableNames: 'centreOnParent'
	poolDictionaries: ''
	category: 'Browser-Dialogs'!

!Prompter class methodsFor: 'instance creation'!

buttonsOk
    ^Array with: Gtk gtkStockOk -> Gtk gtkResponseOk.
!

buttonsOkCancel
    ^Array 
		with: Gtk gtkStockOk -> Gtk gtkResponseOk
		with: Gtk gtkStockCancel -> Gtk gtkResponseCancel.
!

buttonsYesNo
    ^Array 
		with: Gtk gtkStockYes -> Gtk gtkResponseYes
		with: Gtk gtkStockNo -> Gtk gtkResponseNo.
!

centreOnParent
    "This is naughty, but, although we would like the window manager to
     position transients on top of their parents, they sometimes put them
     in far away positions instead, which is rubbish.
     Other options would be to centre on the mouse or in the middle of the
     screen."
    ^centreOnParent ifNil: [ false ]!

centreOnParent: aBoolean
    centreOnParent := aBoolean
!

title: aTitle entry: aLabel buttons: aCollection
    | dlg |
    dlg := self new.
    dlg buildBasic.
    aTitle notNil ifTrue: [ dlg buildTitle: aTitle ].
    aLabel notNil ifTrue: [ dlg buildEntry: aLabel ].
    (dlg buildButtons: aCollection)
		ifFalse: [ dlg buildButtons: self buttonsOk ].
    ^dlg
!
!

!Prompter methodsFor: 'accessing'!

entryText
    ^entry getText
!

entryText: aText
    entry setText: aText
!
!

!Prompter methodsFor: 'initialization'!

getVbox
    | r |
    r := dlg getVbox.
    r isNil ifTrue: [ self error: 'nil' ].
    ^r changeClassTo: GtkVBox.
!

buildBasic
    dlg := GtkDialog new.
    dlg setTitle: 'Prompter'.
    self getVbox setSpacing: 9.
    dlg on: 'response' send: #response:arg1: to: self.
!

buildButtons: aCollection
    aCollection isNil ifTrue: [ ^false ].
    aCollection isEmpty ifTrue: [ ^false ].
    aCollection do: 
		[ :assoc |
		dlg addButton: assoc key responseId: assoc value. ].
    ^true.!

buildEntry: aLabel
    | hbox vbox |
    vbox := self getVbox.
    vbox add: (hbox := (GtkHBox new: false spacing: 3) show; yourself).
    hbox add: ((GtkLabel new: aLabel) show; yourself).
    hbox add: (entry := (GtkEntry new) show; yourself).
    hbox expand: false.
!

buildTitle: aTitle
    | lbl |
    self getVbox add: (lbl := (GtkLabel new: aTitle) show; yourself).
    lbl expand: false.
!
!
