"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

SimpleTreeNode subclass: #TestTreeNode
	instanceVariableNames: 'parent children data selected color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

TestTreeNode subclass: #TestSuiteNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

TestTreeNode subclass: #TestCaseNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

TestTreeNode subclass: #TestMethodNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

TestTreeNode subclass: #NullTestNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Ancilliary'
!

TreeData subclass: #TestData
	instanceVariableNames: 'suites nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Data'
!

PTree subclass: #TestTreeView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!

WindowedView subclass: #TestRunnerView
	instanceVariableNames: 'vbox ebox tree treeMsgs contextMenuItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!

TestTreeNode methodsFor: 'accessing'!

toggleSelected
	selected := (selected ifNotNil: [ :b | b not ] ifNil: [ true ] ).
!

label
	^'?'
!

color
	^color
!

color: aColor
	color := aColor
!

active 
	^(selected notNil and: [ selected ]) 
		ifTrue: [ 1 ]
		ifFalse: [ 0 ].
!

inconsistent
	^selected isNil
		ifTrue: [ 1 ]
		ifFalse: [ 0 ].
!

selected
	^selected
!

selected: aBoolean
	selected := aBoolean
!

selectable
	^1
!
!

TestTreeNode methodsFor: 'walking the tree'!

parentsDo: aBlock
	| p |
	p := self parent.
	[ p notNil ] 
		whileTrue:
			[ aBlock value: p.
			p := p parent. ].
!
!

TestSuiteNode methodsFor: 'accessing'!

label
	^data name ifNil: [ data printString ].
!
!

TestCaseNode methodsFor: 'accessing'!

label
	^data class printString
!
!

TestMethodNode methodsFor: 'accessing'!

label
	^data selector
!

color: aColor
	| p |
	super color: aColor.
	"self parentsDo: 
		[ :each | 
		each color:" 
	
!
!

NullTestNode methodsFor: 'accessing'!

label
	^'? ', data printString
!

color
	^'grey'
!
!

TestData methodsFor: 'initialization'!

addNullNodeAt: aNode for: aUnknown
	| nd |
	nd := NullTestNode new.
	nd data: aUnknown.
	(aNode ifNil: [ self nodes ]) add: nd.
!

addTestMethodNodeAt: aNode for: aTestMethod
	| nd |
	nd := TestMethodNode new.
	nd data: aTestMethod.
	(aNode ifNil: [ self nodes ]) add: nd.
!

addTestCaseNodeAt: aNode for: aTestCase
	| nd |
	nd := TestCaseNode new.
	nd data: aTestCase.
	(aNode ifNil: [ self nodes ]) add: nd.
	aTestCase testSelectors do: 
		[ :ts | self addTestMethodNodeAt: nd for: (aTestCase >> ts) ].
!

addTestSuiteNodeAt: aNode for: aTestSuite
	| nd |
	nd := TestSuiteNode new.
	nd data: aTestSuite.
	(aNode ifNil: [ self nodes ]) add: nd.
	aTestSuite tests do: 
		[ :tc |
		(tc isKindOf: TestSuite) 
			ifTrue:
				[ self addTestSuiteNodeAt: nd for: tc ]
			ifFalse: 
				[ (tc isKindOf: TestCase)
					ifTrue: [ self addTestMethodNodeAt: nd for: tc ]
					ifFalse: [ self addNullNodeAt: nd for: tc ] ] ].
!

buildTestSuiteTree
	self suites do:
		[ :ts |
		self addTestSuiteNodeAt: nil for: ts ].
!

rebuildTree
	nodes := nil.
	self buildTestSuiteTree.
!

defaultSuite
	suites := (OrderedCollection new) 
		add: TestCase buildSuite;
		yourself.
!
!

TestData methodsFor: 'accessing'!

nodes
	nodes isNil ifTrue: [ nodes := OrderedCollection new ].
	^nodes.
!

suites
	suites isNil ifTrue: [ suites := OrderedCollection new ].
	^suites.
!

addSuite: aTestSuite
	self suites add: aTestSuite.
!

clearSuites
	suites notNil ifTrue: [ suites empty ].
!
!

TestData methodsFor: 'change update'!

getCols: aNode
	^aNode notNil
		ifTrue:	
			[ Array 
				with: aNode label 
				with: aNode color 
				with: aNode active
				with: aNode inconsistent ] 
		ifFalse: 
			[ Array 
				with: '?' 
				with: 'black'
				with: 0
				with: 1. ].
!

getNodes: aNode
	^aNode ifNil: [ self nodes ] ifNotNil: [ :nd | nd children ].
!
!

TestRunnerView methodsFor: 'initialization'!

initialize
	(treeMsgs := LookupTable new)
		at: #testSelected put: #nodeSelected;
		at: #testActivated put: #nodeActivated.
!

setup: aData stateChange: aStateChange nodesMsg: aNodesMsg colsMsg: aColsMsg
	self data: aData.
	self 
		stateChange: aStateChange
		updateWith: [ self dataChanged ].
	tree 
		data: aData;
		nodesMsg: aNodesMsg;
		colsMsg: aColsMsg;
		stateChange: aStateChange
			updateWith: [ tree rebuildTree ].
		
!

buildSubsidiaryViews
	super add: (tree := TestTreeView new).
	tree
		buildColumns;
		nodesMsg: #getNodes: colsMsg: #getCols:.
!

buildButtonBar
	| hbox btn |
	(hbox := GtkHBox new: false spacing: 3) show.
	(btn := GtkButton newWithLabel: 'A')
		"on: 'clicked' send: #classBtnCallback to: self;"
		show.
	hbox packStart: btn expand: false fill: false padding: 0.
	(btn := GtkButton newWithLabel: 'B')
		"on: 'clicked' send: #classBtnCallback to: self;"
		show.
	hbox packStart: btn expand: false fill: false padding: 0.
	^hbox
!

buildProgressBar
	| bar |
	(bar := GtkFrame new: 'Progress') show.
	^bar.
!

buildCountsBar
	| bar |
	(bar := GtkFrame new: 'Counts') show.
	^bar.
!

buildResultsBox
	| bar |
	(bar := GtkFrame new: 'Results') show.
	^bar.
!

buildResultsDetailBox
	| bar |
	(bar := GtkFrame new: 'Detail') show.
	^bar.
!

buildWidgets
	| al hbox lbl sb btn |
	super buildWidgets.
	
    (ebox := GtkEventBox new) 
		setName: 'testEbox';
		show.
	
	(vbox := GtkVBox new: false spacing: 3) show.
	ebox add: vbox.

	vbox packStart: self buildButtonBar expand: false fill: false padding: 9.
	vbox packEnd: self buildResultsDetailBox expand: false fill: false padding: 9.
	vbox packEnd: self buildResultsBox expand: false fill: false padding: 9.
	vbox packEnd: self buildProgressBar expand: false fill: false padding: 9.

	self hookupPopupMenuTo: ebox.
!

attachViews		
	vbox packStart: tree external expand: true fill: true padding: 0.
	"For some reason, doing these in the opposite order stops the popup
	 menu from appearing."
    self hookupPopupMenuTo: tree treeView.
	tree treeView setName: 'methodTree'.
!
!

TestRunnerView methodsFor: 'change update'!

dataChanged
	^self
!
!

TestRunnerView methodsFor: 'accessing'!

external
	^ebox
!
!

TestRunnerView methodsFor: 'events'!

runOneCallback
	self notYetImplemented
!

runAllCallback
	self notYetImplemented
!
!

TestTreeView methodsFor: 'accessing'!

treeStore
	(tree isKindOf: GtkTreeStore) 
		ifTrue: [ ^tree ]
		ifFalse: [ ^tree castTo: GtkTreeStore ].
!
!

TestTreeView methodsFor: 'events'!

rowToggledCallback: aSender path: aPath
	| iter idx ety |
	iter := GtkTreeIter type new.
	tree getIterFromString: iter pathString: aPath.
	idx := tree getOop: iter column: 0.
	ety := entries at: idx.
	ety toggleSelected.
	"FIXME - This column index is not reliable"
	(self treeStore)
		setOop: iter column: 3 value: (ety active);
		setOop: iter column: 4 value: (ety inconsistent).
!
!

TestTreeView methodsFor: 'initialization'!

buildColumns
    "Convenience method for creating simple tree views"
    | col rend typs n |

	n := 4.

    typs := (CArrayCType elementType: CIntType numberOfElements: n + 1) new.
    typs at: 0 put: (GValue typeFromName: 'gint').
    1 to: 2 do: 
		[ :i | typs at: i put: (GValue typeFromName: 'gchararray'). ].
	3 to: 4 do: 
		[ :i | typs at: i put: (GValue typeFromName: 'gint'). ].

	tree := GtkTreeStore newv: n + 1 types: typs.

    treeView setModel: tree.

   	col := GtkTreeViewColumn new.
	treeView insertColumn: col position: -1. "-1 => append"
	col setTitle: 'Test'.
	
	col packStart: (rend := GtkCellRendererToggle new) expand: false.
	col addAttribute: rend attribute: 'active' column: 3.
	col addAttribute: rend attribute: 'inconsistent' column: 4.
	rend on: 'toggled' send: #rowToggledCallback:path: to: self.
	
	col packStart: (rend := GtkCellRendererText new) expand: false.
	col addAttribute: rend attribute: 'cell-background' column: 2.

	col packStart: (rend := GtkCellRendererText new) expand: true.
	col addAttribute: rend attribute: 'text' column: 1.
!
!

WindowFactory class methodsFor: 'window creation'!

buildTestRunnerWindow
	| vw d wnd |
	d := TestData new.
	vw := TestRunnerView new.
	vw
		setup: d 
		stateChange: #dataChanged 
		nodesMsg: #getNodes:
		colsMsg: #getCols:.

	d 
		defaultSuite;	
		rebuildTree; 
		changeState: #dataChanged.
		
	wnd := self buildWindowFor: vw title: 'SUnit Test Browser'.
	wnd	external resize: 300 height: 220.
	^wnd
!
!
