"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

Namespace current: GTK.GUI.Mumble!

Object subclass: #WindowFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mumble-Main'!

!WindowFactory class methodsFor: 'window creation'!

buildWindowFor: aView title: aTitle
	| wnd border |
	(wnd := BrowserWindowView new)
		title: aTitle.

	(border := PCustom new)
		buildBlock: 
			[ :v | 
			v widget: 
				((GtkTable new: 1 columns: 1 homogeneous: false) 
					show; 
					yourself) ];
		attachBlock:
			[ :v :c | 
			v widget 
				attach: c external 
				leftAttach: 0
				rightAttach: 1
				topAttach: 0
				bottomAttach: 1
				xoptions: (Gtk gtkExpand + Gtk gtkFill)
				yoptions: (Gtk gtkExpand + Gtk gtkFill)
				xpadding: 4
				ypadding: 5 ].
	wnd add: border.
	border add: aView.
	^wnd
!

buildWorkspaceWindow: aObject
	| ed wv wnd |
	ed := EvaluatorData new.
	wv := WorkspaceView new.
	wv setup: ed stateChange: #targetChanged.
	ed target: aObject.
	wnd := self 
		buildWindowFor: wv 
		title: 'Workspace: ', (WindowView safeTitleString: aObject displayString).
	wnd	external resize: 280 height: 200.
	^wnd
!

buildInspectorWindow: aObject
	| id iv wnd |
	id := InspectorData new.
	(iv := InspectorView new) setup: id stateChange: #targetChanged.
	wnd := self 
		buildWindowFor: iv 
		title: 'Inspecting ', (WindowView safeTitleString: aObject displayString).
	wnd external resize: 300 height: 200.
	id target: aObject.
	^wnd
!

buildNamespacesWindow
	| nsv d wnd |
	(d := NamespacesData new) changeState: #namespacesChanged.
	nsv := NamespacesView new.
	nsv 
		setup: d 
		stateChange: #namespacesChanged 
		nodesMsg: #getNodes:
		colsMsg: #getCols:;
		
		on: #namespaceActivated
		send: #buildNamespaceWindowCallback:data: 
		to: self.

	wnd := self buildWindowFor: nsv title: 'Namespaces'.
	wnd	external resize: 200 height: 150.
	^wnd
!

buildNamespaceWindow: aNamespace
	| nsv d wnd |
	d := NamespaceData new.
	nsv := NamespaceView new.
	nsv 
		setup: d 
		stateChange: #namespaceChanged 
		nodesMsg: #getNodes:
		colsMsg: #getCols:;
		
		on: #classActivated
		send: #buildClassWindowCallback:data: 
		to: self.
		
	d displayNamespace: aNamespace.
	wnd := self buildWindowFor: nsv title: aNamespace displayString.
	wnd	external resize: 200 height: 500.
	^wnd
!

buildNamespaceWindowCallback: aSender data: aNamespace
	(self buildNamespaceWindow: aNamespace) show.
	Primitive updateViews.
!

buildNamespaceWindow
	^self buildNamespaceWindow: Smalltalk
!

buildClassWindow: aClass
	| cv d wnd |
	d := ClassData new.
	cv := ClassView new.
	cv 
		setup: d 
		stateChange: #classChanged
		nodesMsg: #getNodes:
		colsMsg: #getCols: ;
		
		on: #methodActivated
		send: #buildMethodWindowCallback:data: 
		to: self;

		on: #superBtnClicked 
		update: self 
		with: 
			[ (self buildClassWindow: aClass superclass) show.
			Primitive updateViews ];

		on: #newMethodClicked
		send: #buildNewMethodWindowCallback:assoc:
		to: self.

	d displayClass: aClass.
	wnd := self buildWindowFor: cv title: aClass displayString.
	wnd	external resize: 380 height: 350.
	^wnd
!

buildClassWindowCallback: aSender data: aClass
	(self buildClassWindow: aClass) show.
	Primitive updateViews.
!

buildMethodWindow: aMethod class: aClass category: aCategory
	| mv d wnd |
	d := MethodData new.
	mv := MethodView new.
	mv 
		data: d;
		methodChange: #methodChanged;
		
		on: #classBtnClicked 
		update: self 
		with: 
			[ (self buildClassWindow: d methodClass) show.
			Primitive updateViews ].

	(wnd := self buildWindowFor: mv title: 'Method Detail')
		stateChange: #methodChanged 
		updateWith: [ wnd title: d description ].
	wnd external resize: 380 height: 250.
	
	d displayMethod: aMethod class: aClass category: aCategory.
	^wnd
!

buildMethodWindow: aMethod
	^self 
		buildMethodWindow: aMethod
		class: aMethod methodClass
		category: aMethod methodCategory
!

buildMethodWindowCallback: aSender data: aMethodNode
	| cm |
	((cm := aMethodNode data) isKindOf: CompiledMethod) ifFalse: [ ^self ].
	(self buildMethodWindow: cm) ifNotNil: 
		[ :w | w show ].
	Primitive updateViews.
!

buildMethodWindowClass: aClass category: aCategory
	^self 
		buildMethodWindow: nil
		class: aClass
		category: aCategory
!

buildNewMethodWindowCallback: aSender assoc: aAssoc
	(self buildMethodWindowClass: aAssoc key category: aAssoc value) 
		ifNotNil: 
			[ :w | w show].
	Primitive updateViews.
!

buildMethodSetWindow: aMethodList
	| msv wnd |
	msv := MethodSetView new.
	msv 
		setup: aMethodList
		listChange: #listChanged
		descChange: #descriptionChanged.
	wnd := self 
		buildWindowFor: msv
		title: msv title.
	wnd	external resize: 240 height: 300.
	^wnd
!

buildTranscriptWindow
	| wnd |
	wnd := self 
		buildWindowFor: TranscriptView new 
		title: 'Transcript'.
	wnd	external resize: 200 height: 280.
	^wnd
!
!

