"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"


WindowedView subclass: #WorkspaceView
	instanceVariableNames: 'vbox object eval titleEBox objectLbl changeLogBtns'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Browser-Windows'!

!WorkspaceView methodsFor: 'initialization'!

buildSubsidiaryViews
	self add: (eval := EvaluatorView new).
!

buildWidgets
    | btn hbox lbl hbox2 |

    super buildWidgets.

	(vbox := GtkVBox new: false spacing: 3) show.
	
	titleEBox := GtkEventBox new. "No show"
    vbox packStart: titleEBox expand: false fill: false padding: 0.
	
	(hbox := GtkHBox new: false spacing: 3) show.
	titleEBox add: hbox.
	self hookupPopupMenuTo: titleEBox.

	(objectLbl := GtkLabel new: object printString) show.
	hbox packStart: objectLbl expand: false fill: false padding: 0.

    "evalBtns := ((GtkAlignment new: 1.0 yalign: 0.5 xscale: 0.7 yscale: 1.0) show; yourself).
    hbox1 packEnd: evalBtns expand: false fill: false padding: 0.

    evalBtns add: ((hbox2 := GtkHBox new: false spacing: 3) show; yourself). 

    hbox2 add: ((btn := GtkButton newWithLabel: 'Do it') show; yourself).
    btn on: 'clicked' send: #doItCallback to: self .

    hbox2 add: ((btn := GtkButton newWithLabel: 'Print it') show; yourself).
    btn on: 'clicked' send: #printItCallback to: self .

    hbox2 add: ((btn := GtkButton newWithLabel: 'Get it') show; yourself).
    btn on: 'clicked' send: #getItCallback to: self ."

    "FIXME
	changeLogBtns := GtkAlignment new: 0.5 yalign: 0.5 xscale: 0.7 yscale: 1.0. 
    vbox add: changeLogBtns.    
    changeLogBtns expand: false.
    changeLogBtns add: ((hbox := GtkHBox new: false spacing: 3) show; yourself).
    hbox add: ((btn := GtkButton newWithLabel: 'Log') show; yourself).
    btn on: 'clicked' send: #logCallback to: self.
    hbox add: ((btn := GtkButton newWithLabel: 'Close') show; yourself).
    btn on: 'clicked' send: #close to: self.	
    text on: 'insert-text' send: #discardCallback to: self.
    text on: 'delete-range' send: #discardCallback to: self."
!

attachViews
    vbox packStart: eval external expand: true fill: true padding: 0.
!

buildPopupMenuItems
	super buildPopupMenuItems.
	popupMenuItems
		section: 'Templates' add: 'New class template' sending: #newClassTemplate to: self;
		section: 'Filing' add: 'Load file...' sending: #promptLoadFile to: self;
		section: 'Filing' add: 'Save to file...' sending: #promptSaveFile to: self;
		section: 'View' add: 'Use scrollbars' sending: #useScrollbarsCallback to: self.
!

setup: aData stateChange: aStateChange
	self 
		data: aData;
		stateChange: aStateChange updateWith: [ self targetChanged ].
	eval setup: aData stateChange: aStateChange.
!

targetChanged
	| t s |
	t := data target.
	t isNil 
		ifTrue: 
			[ titleEBox hide. ]
		ifFalse: 
			[ titleEBox show.
			s := data target displayString contractTo: 75.
			s := s reject: [ :ch | ch = Character nl ].
			s := Pango quoteMarkup: s.
			object notNil ifTrue: [ s := '<big>', s, '</big>' ].
			objectLbl setMarkup: s. ].
!
!

!WorkspaceView methodsFor: 'accessing'!

external
	^vbox
!

!WorkspaceView methodsFor: 'events'!

doItCallback
    eval doItCallback
!

getItCallback
	eval getItCallback.
!

promptLoadFile
    | dlg |
    dlg := GtkFileSelection new: 'Load file'.
	[ (dlg run = Gtk gtkResponseOk) 
		ifTrue: 
			[data loadFile: dlg getFilename ] ] 
		ensure: 
			[ dlg destroy ].
!

logCallback
    "FIXME
    CurrentChanges expressionChange: object -> text getText.
    changeLogBtns hide."
!

newClassTemplate
	eval text 
		setText: 
'ParentClass subclass: #Subclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''Uncategorized''' 
		len: -1.
!

printItCallback
    eval printItCallback
!

promptSaveFile
    | dlg |
    dlg := FileSelection new: 'Save file'.
    (dlg run = Gtk gtkResponseOk) 
		ifTrue: 
			[ (File name: dlg getFilename) writeStream 
				nextPutAllFlush: eval text getText. ].
    dlg destroy.
!

useScrollbarsCallback
    | pol |
    pol := Gtk gtkPolicyAutomatic.
    eval scroll setPolicy: pol vscrollbarPolicy: pol.
!
!
